from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param.montior_item import MonitorItemParam
from pyctyun.response.monitor_item import MonitorItemResponse


class MonitorItemApi(CtapiBaseClient):
    
    def get_monitor_items(self, param: MonitorItemParam):
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request("/v4/monitor/query-monitor-items",
                                    params,
                                    method='GET')
        return MonitorItemResponse(**data)



from typing import Optional

from pydantic import Field, field_validator, StrictStr

from pyctyun.rules.rule import PaginationGetParamModel, check_ip


class ListEipParam(PaginationGetParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    ids: Optional[list[StrictStr]] = None
    status: Optional[StrictStr] = None
    ip_type: Optional[StrictStr] = Field('ipv4', alias='ipType')
    eip_type: Optional[StrictStr] = Field('normal', alias='eipType')
    ip: Optional[StrictStr] = None
    include_device: bool = Field(False, alias='includeDevice')
    
    _check_address = field_validator('ip')(check_ip)
    
    @field_validator('status')
    def check_status(cls, value):
        status = ['ACTIVE', 'DOWN', 'FREEZING', 'EXPIRED']
        if value and value not in status:
            raise ValueError("only support ACTIVE / DOWN / EXPIRED / FREEZING")
        return value
    
    @field_validator('ip_type')
    def check_ip_type(cls, value):
        if value and value not in ['ipv4', 'ipv6']:
            raise ValueError('should be ipv4 or ipv6')
        return value
    
    @field_validator('eip_type')
    def check_eip_type(cls, value):
        if value and value not in ['normal', 'cn2']:
            raise ValueError('should be normal or cn2')
        return value


import unittest

from src.rules import rule


class TestRules(unittest.TestCase):
    
    def test_check_cycle_count(self):
        rule.check_cycle_count(1, {"cycle_type": 'year'})
        rule.check_cycle_count(1, {"cycle_type": 'month'})
        rule.check_cycle_count(1, {'cycle_type': 'on_demand'})
        
        with self.assertRaises(ValueError):
            rule.check_cycle_count(4, {"cycle_type": 'year'})
            
        with self.assertRaises(ValueError):
            rule.check_cycle_count(12, {"cycle_type": 'month'})
            
    def test_check_dns_name(self):
        rule.check_dns_name("a.b.c")
        with self.assertRaises(ValueError):
            rule.check_dns_name("a" * 255)
            
        with self.assertRaises(ValueError):
            a = 'a' * 64
            rule.check_dns_name(f'{a}.c')
            
    def test_check_aaaa(self):
        rule.check_aaaa("E24D:0D46:B5D1:FEBE:1CD8:0BD6:D8A3:6C27")
        with self.assertRaises(ValueError):
            rule.check_aaaa("192.168.1.1")
        
    def test_check_uuid(self):
        rule.check_uuid("a-b-c-d")
        
        with self.assertRaises(ValueError):
            rule.check_uuid("AAAAA")
            
    def test_check_uuids(self):
        rule.check_uuids(["a-b-c-d"])
        with self.assertRaises(ValueError):
            rule.check_uuids(["AAAAA"])
            
    def test_check_email(self):
        rule.check_email("foo@bar.com")
        with self.assertRaises(ValueError):
            rule.check_email("sdsdsds")



from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param import dhcp
from pyctyun.response import dhcp as dhcp_response


class DhcpApi(CtapiBaseClient):
    
    def create_dhcp(self, param: dhcp.CreateDhcpoptionSetsParam) -> dhcp_response.DhcpOptionSetIDResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/dhcpoptionsets/create', params=params, method='POST')
        return dhcp_response.DhcpOptionSetIDResponse(**data)
    
    def delete_dhcp(self, param: dhcp.DeleteDhcpoptionSetsParam) -> dhcp_response.DhcpOptionSetIDResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/dhcpoptionsets/delete', params=params, method='POST')
        return dhcp_response.DhcpOptionSetIDResponse(**data)
    
    def update_dhcp(self, param: dhcp.UpdateDhcpoptionSetsParam) -> dhcp_response.DhcpOptionSetIDResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/dhcpoptionsets/update', params=params, method='POST')
        return dhcp_response.DhcpOptionSetIDResponse(**data)
    
    def show_dhcp(self, param: dhcp.ShowDhcpoptionSetsParam) -> dhcp_response.ShowDhcpResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/dhcpoptionsets/show', params=params, method='GET')
        return dhcp_response.ShowDhcpResponse(**data)
    
    def list_dhcp(self, param: dhcp.DhcpoptionListVpcParam) -> dhcp_response.ListDhcpResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/dhcpoptionsets/query', params=params, method='GET')
        return dhcp_response.ListDhcpResponse(**data)
    
    def dhcp_associate_vpc(self, param: dhcp.DhcpoptionWithVpcParam) -> dhcp_response.DhcpOptionSetIDResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/dhcpoptionsets/dhcp_associate_vpc', params=params, method='POST')
        return dhcp_response.DhcpOptionSetIDResponse(**data)
    
    def dhcp_disassociate_vpc(self, param: dhcp.DhcpoptionWithVpcParam) -> dhcp_response.DhcpOptionSetIDResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/dhcpoptionsets/dhcp_disassociate_vpc', params=params, method='POST')
        return dhcp_response.DhcpOptionSetIDResponse(**data)
    
    def dhcp_replace_vpc(self, param: dhcp.DhcpoptionWithVpcParam) -> dhcp_response.DhcpOptionSetIDResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/dhcpoptionsets/dhcp_replace_vpc', params=params, method='POST')
        return dhcp_response.DhcpOptionSetIDResponse(**data)
    
    def dhcp_list_unbind_vpc(self, param: dhcp.DhcpoptionListUnbindVpcParam) -> dhcp_response.UnbindVpcResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/dhcpoptionsets/dhcp_list_unbind_vpc', params=params, method='GET')
        return dhcp_response.ListDhcpResponse(**data)
    
    def dhcp_list_vpc(self, param: dhcp.DhcpoptionListVpcParam) -> dhcp_response.BindVpcResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/dhcpoptionsets/dhcp_list_vpc', params=params, method='GET')
        return dhcp_response.BindVpcResponse(**data)



from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param import subnet
from pyctyun.response.base import BaseResponse
from pyctyun.response.subnet import CreateSubnetResponse, NewListSubnetResponse, ListSubnetResponse, ShowSubnetResponse, \
    CheckIpAvailableResponse


class SubnetApi(CtapiBaseClient):
    
    def create_subnet(self, param: subnet.CreateSubnetParam) -> CreateSubnetResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/create-subnet', params=params, method='POST')
        return CreateSubnetResponse(**data)
    
    def delete_subnet(self, param: subnet.DeleteSubnetParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/delete-subnet', params=params, method='POST')
        return BaseResponse(**data)
    
    def enable_ipv6(self, param: subnet.SubnetEnableIPv6Param) -> BaseResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/subnet-enable-ipv6', params=params, method='POST')
        return BaseResponse(**data)
    
    def replace_acl(self, param: subnet.SubnetReplaceACLParam) -> BaseResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/replace-subnet-acl', params=params, method='POST')
        return BaseResponse(**data)
    
    def replace_route_table(self, param: subnet.SubnetReplaceRouteTableParam) -> BaseResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/replace-subnet-route-table', params=params, method='POST')
        return BaseResponse(**data)
    
    def disassociate_acl(self, param: subnet.SubnetDisassociateACLParam) -> BaseResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/disassociate-subnet-acl', params=params, method='POST')
        return BaseResponse(**data)
    
    def disassociate_route_table(self, param: subnet.SubnetDisassociateRouteTableParam) -> BaseResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/subnet-disassociate-route-table', params=params, method='POST')
        return BaseResponse(**data)
    
    def show_subnet(self, param: subnet.ShowSubnetParam) -> ShowSubnetResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/query', params=params, method='GET')
        return ShowSubnetResponse(**data)
    
    def new_list_subnets(self, param: subnet.ListSubnetsParam) -> NewListSubnetResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/new-list-subnet', params=params, method='GET')
        return NewListSubnetResponse(**data)
    
    def list_subnets(self, param: subnet.ListSubnetsParam) -> ListSubnetResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/list-subnet', params=params, method='GET')
        return ListSubnetResponse(**data)
    
    def check_ip_avaliable(self, param: subnet.CheckIpAvaliableParam) -> CheckIpAvailableResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/check-ip-avaliable', params=params, method='GET')
        return CheckIpAvailableResponse(**data)



from typing import Optional, List

from pydantic import Field, field_validator, StrictStr

from pyctyun.rules.rule import check_description, BaseParamModel, check_ips, \
    check_name, PaginationGetParamModel, check_domain_name, check_ipv4_map_ipv6_addr


class CreateDhcpoptionSetsParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    name: StrictStr
    domain_name: StrictStr = Field(alias='domainName', min_length=1)
    description: Optional[StrictStr]
    dns_list: List[StrictStr] = Field(alias='dnsList', min_items=1, max_items=4, min_length=1)
    
    _check_description = field_validator('description')(check_description)
    _check_name = field_validator('name')(check_name)
    _check_domain_name = field_validator('domain_name')(check_domain_name)
    
    @field_validator("dns_list", allow_reuse=True)
    def check_dns_list(cls, v):
        check_ips(v)
        for i in v:
            check_ipv4_map_ipv6_addr(i)
        return v


class DeleteDhcpoptionSetsParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    dhcpoptionsets_id: StrictStr = Field(alias='dhcpOptionSetsID', min_length=1)


class UpdateDhcpoptionSetsParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    dhcpoptionsets_id: StrictStr = Field(alias='dhcpOptionSetsID', min_length=1)
    name: Optional[StrictStr]
    domain_name: Optional[StrictStr] = Field(alias='domainName', min_length=1)
    description: Optional[StrictStr]
    dns_list: Optional[List[StrictStr]] = Field(alias='dnsList', min_items=1, max_items=4, min_length=1)
    
    _check_description = field_validator('description')(check_description)
    _check_name = field_validator('name')(check_name)
    _check_domain_name = field_validator('domain_name')(check_domain_name)
    
    @field_validator("dns_list", allow_reuse=True)
    def check_dns_list(cls, v):
        check_ips(v)
        for i in v:
            check_ipv4_map_ipv6_addr(i)
        return v


class QueryDhcpoptionSetsParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    query_content: Optional[StrictStr] = Field(alias='queryContent')


class ShowDhcpoptionSetsParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    dhcpoptionsets_id: StrictStr = Field(alias='dhcpOptionSetsID', min_length=1)


class DhcpoptionWithVpcParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    dhcpoptionsets_id: StrictStr = Field(alias='dhcpOptionSetsID', min_length=1)
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)


class DhcpoptionListVpcParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    dhcpoptionsets_id: StrictStr = Field(alias='dhcpOptionSetsID', min_length=1)


class DhcpoptionListUnbindVpcParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)



class CtapiException(Exception):
    name = 'ctapi base exception'

    def __init__(self, status_code, reason):
        self.status_code = status_code
        self.reason = reason

    def __str__(self):
        return '{name}({code}, {reason})'.format(
                name=self.name,
                code=self.status_code,
                reason=self.reason)
    

class TimeoutError(CtapiException):
    name = 'timeout error'
    

class IamApiError(CtapiException):
    name = 'iam api error'


class ClientRequestError(CtapiException):
    name = 'client request error'


class InternalServerError(CtapiException):
    name = 'internal server error'



from typing import Optional, List

from pydantic import Field, field_validator, StrictStr, conint, StrictInt

from pyctyun.rules.rule import check_description, BaseParamModel, check_cidr, check_name, PaginationGetParamModel


class PrefixListRule(BaseParamModel):
    cidr: StrictStr = Field(min_length=1)
    description: Optional[StrictStr]

    _check_cidr = field_validator('cidr')(check_cidr)
    _check_description = field_validator('description')(check_description)


class CreatePrefixListParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    name: StrictStr
    limit: conint(strict=True, ge=1, le=200)
    description: Optional[StrictStr]
    address_type: StrictInt = Field(alias='addressType')
    prefix_list_rules: List[PrefixListRule] = Field(alias='prefixListRules', min_items=1, max_items=200)

    _check_name = field_validator('name')(check_name)
    _check_description = field_validator('description')(check_description)

    @field_validator('address_type')
    def check_address_type(cls, value: int):
        if value and value not in (4, 6):
            raise ValueError('only support 4 / 6')
        return value


class QueryPrefixListParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    query_content: Optional[StrictStr] = Field(alias='queryContent', min_length=1)
    prefix_list_id: Optional[StrictStr] = Field(alias='prefixListID', min_length=1)


class ShowPrefixListParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    prefix_list_id: StrictStr = Field(alias='prefixListID', min_length=1)


class UpdatePrefixListParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    prefix_list_id: StrictStr = Field(alias='prefixListID', min_length=1)
    name: Optional[StrictStr]
    description: Optional[StrictStr]

    _check_name = field_validator('name', allow_reuse=True)(check_name)
    _check_description = field_validator('description', allow_reuse=True)(check_description)


class DeletePrefixListParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    prefix_list_id: StrictStr = Field(alias='prefixListID', min_length=1)


class CreatePrefixListRuleParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    prefix_list_id: StrictStr = Field(alias='prefixListID', min_length=1)
    prefix_list_rules: List[PrefixListRule] = Field(alias='prefixListRules', min_items=1, max_items=200)


class UpdatePrefixListRuleParam(PrefixListRule):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    prefix_list_id: StrictStr = Field(alias='prefixListID', min_length=1)
    prefix_list_rule_id: StrictStr = Field(alias='prefixListRuleID', min_length=1)


class DeletePrefixListRuleParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    prefix_list_id: StrictStr = Field(alias='prefixListID', min_length=1)
    prefix_list_rule_ids: StrictStr = Field(alias='prefixListRuleIDs', min_length=1)


class ClonePrefixListParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    dest_region_id: StrictStr = Field(alias='destRegionID', min_length=1)
    prefix_list_id: StrictStr = Field(alias='prefixListID', min_length=1)
    name: StrictStr
    limit: Optional[conint(strict=True, ge=1, le=200)]
    description: Optional[StrictStr]

    _check_name = field_validator('name')(check_name)
    _check_description = field_validator('description')(check_description)


class ListBindResourceParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    prefix_list_id: StrictStr = Field(alias='prefixListID', min_length=1)



from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param import vpc
from pyctyun.response.base import BaseResponse
from pyctyun.response.vpc import CreateVpcResponse, ListVpcResponse, ShowVpcResponse


class VpcApi(CtapiBaseClient):
    
    def create_vpc(self, param: vpc.CreateVPCParam) -> CreateVpcResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/create', params=params, method='POST')
        return CreateVpcResponse(**data)
    
    def delete_vpc(self, param: vpc.DeleteVPCParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/delete', params=params, method='POST')
        return BaseResponse(**data)
    
    def describe_vpcs(self, param: vpc.ListVPCParam) -> ListVpcResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/new-list', params=params, method='GET')
        return ListVpcResponse(**data)
    
    def show_vpc(self, param: vpc.ShowVPCParam) -> ShowVpcResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/query', params=params, method='GET')
        return ShowVpcResponse(**data)
    
    def update_vpc(self, param: vpc.UpdateVPCParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/update', params=params, method='POST')
        return BaseResponse(**data)
    
    def update_vpc_ipv6(self, param: vpc.UpdateVPCIPv6StatusParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/update-ipv6-status', params=params, method='POST')
        return BaseResponse(**data)



import ipaddress
import re
from typing import List, Optional

from email_validator import EmailNotValidError, EmailSyntaxError, validate_email
from pydantic import BaseModel, conint, Field, StrictStr

name_regex = re.compile(
    r'^([a-zA-Z]|[\u4e00-\u9fff])([a-zA-Z0-9_\-]|[\u4e00-\u9fff]){1,31}$')
vpce_service_name_regex = re.compile(
    r'^(([a-zA-Z0-9]|[\u4e00-\u9fff])([a-zA-Z0-9_\-]|[[\u4e00-\u9fff]]){0,63})(\.([a-zA-Z0-9]|[[\u4e00-\u9fff]])([a-zA-Z0-9_\-]|[[\u4e00-\u9fff]]){0,63})*$')
desc_regex = re.compile(
    r"^([~!@#$%^&*()_\-+= <>?:\"{}|,.\/;'[\]·~！@#￥%……&*（） ——\-+={}|《》？：“”【】、；‘'，。、]|[a-zA-Z0-9]|[\u4e00-\u9fff]){0,128}$")
uuid_regex = re.compile(r'^[a-z0-9][a-z0-9-]{1,35}$')
vpcx_device_regex = re.compile(r'^[a-zA-Z0-9_-]{1,36}$')
email_rule = re.compile(r"^[\w\.\+\-]+\@[\w]+\.[a-z]{2,3}$")
dns_name_regex = re.compile(
    r'^[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(?<!-)(\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})+(?<!-)$')
dns_name_wild_regex = re.compile(
    r'^(\*)(\.[a-zA-Z0-9]([a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?)+$')
mx_value_regex = re.compile(
    r'^[0-9]{1,5}\s[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})*$')
txt_value_regex = re.compile(
    r'^(\"[a-zA-Z0-9~\!\@\#\$\%\^\&\*\(\)\_\+\-\=\{\}\|\[\]\:\;\'\,\.\/\<\>\?\s]{0,255}\")(\s\"[a-zA-Z0-9\~\!\@\#\$\%\^\&\*\(\)\_\+\-\=\{\}\|\[\]\:\;\'\,\.\/\<\>\?\s]{0,255}\")*$')

host_record_regex = re.compile(
    r'^(\*\.)?[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(?<!-)(\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})*(?<!-)$')
domain_name_regex = re.compile(
    r'^([a-zA-Z0-9][a-zA-Z0-9\-]{0,62})(\.[a-zA-Z0-9][a-zA-Z0-9\-]{0,62})+$')
elb_http_expected_codes_regex = re.compile(
    r'^(\d{3}(\s*,\s*\d{3})*)$|^(\d{3}-\d{3})$')
srv_regex = re.compile(
    r'^[0-9]{1,5}\s[0-9]{1,5}\s[0-9]{1,5}\s[a-zA-Z0-9][-a-zA-Z0-9]{0,62}(\.[a-zA-Z0-9][-a-zA-Z0-9]{0,62})*$')
http_url_path_regex = re.compile(r'^(/[0-9a-zA-Z\-_./=]*)?$')
mac_regex = re.compile(
    r'^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$|^([0-9A-Fa-f]{4}\.){2}([0-9A-Fa-f]{4})$')


class BaseParamModel(BaseModel):
    class Config:
        str_strip_whitespace = True


class GetQuotaParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)


class PaginationGetParamModel(BaseParamModel):
    page_number: Optional[conint(ge=1)] = Field(1, alias='pageNumber')
    page_size: conint(ge=1, le=50) = Field(10, alias='pageSize')


class PaginationPostParamModel(BaseParamModel):
    page_number: Optional[conint(strict=True, ge=1)] = Field(
        alias='pageNumber')
    page_size: conint(strict=True, ge=1, le=50) = Field(10, alias='pageSize')


def check_dns_name(name: str | None) -> str | None:
    if name is not None:
        if len(name) == 0:
            return name
        
        if not 1 <= len(name) <= 254:
            raise ValueError("length should be 1 - 254")
        
        ary = name.split('.')
        for i in ary:
            if not 1 <= len(i) <= 63:
                raise ValueError(
                    "each part of domain name length should be 1 - 63")
        
        if not dns_name_regex.match(name) and not dns_name_wild_regex.match(name):
            raise ValueError(
                "must contain dot, must not start with or end with hyphen, support letter, digit, hyphen, dot")
    return name


def check_ptr(name: str | None) -> str | None:
    if name is not None:
        match = dns_name_regex.match(name)
        if not match or len(name) > 254:
            raise ValueError(
                f"Zone record PTR type value {name} was wrong format")
    return name


def check_srv(name: str | None) -> str | None:
    if name is not None:
        if not srv_regex.match(name):
            raise ValueError(
                f'Zone record SRV type value {name} wrong format.')
        
        tmp_ary = name.split(" ")
        if len(tmp_ary) != 4:
            raise ValueError(
                f'Zone record SRV type value {name} wrong format.')
        
        priority, weight, port, domain_name = tmp_ary
        if not priority.isdigit() or not 1 <= int(priority) <= 65535:
            raise ValueError(
                'Zone record SRV type value priority must be [0, 65535] range.')
        
        if not weight.isdigit() or not 1 <= int(weight) <= 65535:
            raise ValueError(
                'Zone record SRV type value weight must be [0, 65535] range.')
        
        if not port.isdigit() or not 1 <= int(port) <= 65535:
            raise ValueError(
                'Zone record SRV type value port must be [0, 65535] range.')
        
        is_match = dns_name_regex.match(domain_name)
        if not is_match or len(domain_name) > 254:
            raise ValueError(
                f"Zone record SRV type value {domain_name} was wrong format")
    return name


def check_aaaa(name: str | None) -> str | None:
    if name is not None:
        tmp = ipaddress.ip_address(name)
        if tmp.version != 6:
            raise ValueError(
                'Zone record AAAA type value must be IPv6 format.')
    return name


def check_uuid(value: str) -> str:
    if value and not uuid_regex.match(value):
        raise ValueError(f"{value} format error")
    return value


def check_uuids(value: list[str]) -> list[str] | None:
    if value is not None:
        for i in value:
            check_uuid(i)
    return value


def check_email(email: str) -> str:
    if email is not None:
        try:
            validate_email(email, check_deliverability=False)
        except (EmailSyntaxError, EmailNotValidError):
            raise ValueError(f"{email} format error")
    return email


def check_vpce_service_name(name: str):
    if name is not None:
        if name.startswith('http:') or name.startswith('https:'):
            raise ValueError("should not start with http: or https:")
        if not vpce_service_name_regex.match(name):
            raise ValueError(
                "format error：only support en letter、chinese、-、_、digits, and should starts with letter / chinese")
    return name


def check_name(name: str) -> str:
    if name is not None:
        if name.startswith('http:') or name.startswith('https:'):
            raise ValueError("should not start with http: or https:")
        if not 2 <= len(name) <= 32:
            raise ValueError("length should be  2 - 32")
        if not name_regex.match(name):
            raise ValueError(
                "format error：only support en letter、chinese、-、_、digits, and should starts with letter / chinese")
    return name


def check_pay_voucher_price(price: str) -> str:
    try:
        tmp = float(price)
        if tmp < 0:
            raise ValueError("must bigger than 0")
        if "." in price:
            part = price.split(".")[1]
            if len(part) > 2:
                raise ValueError("two decimal places")
        return price
    except ValueError:
        raise


def check_oa_type(oa_type: str) -> str:
    if oa_type is not None:
        if oa_type not in ('tcp_option', 'proxy_protocol', 'close'):
            raise ValueError(
                "only support close / tcp_option / proxy_protocol")
    return oa_type


def check_host_record(host_record, v):
    if host_record != "" and host_record != '*':
        if len(v) > 256:
            raise ValueError("length should less than equal to 256")
        
        if not host_record_regex.match(v):
            raise ValueError(f'zone record host record {v} was wrong format')


def check_record_type(value: str) -> str:
    if value is not None:
        if value.upper() not in ('A', 'AAAA', 'TXT', 'CNAME', 'MX'):
            raise ValueError("only support A / AAAA / TXT / CNAME / MX")
    return value


def check_certificate_type(certificate_type: str) -> str:
    if certificate_type is not None:
        if certificate_type not in ('Server', 'Ca'):
            raise ValueError('only support Server / Ca')
    return certificate_type


def check_elb_resource_type(resource_type: str) -> str:
    if resource_type is not None:
        if resource_type not in ('external', 'internal'):
            raise ValueError('only support external / internal')
    return resource_type


def check_proxy_pattern(proxy_pattern: str) -> str:
    if proxy_pattern is not None:
        if proxy_pattern.lower() not in ('zone', 'record'):
            raise ValueError('only support zone / record')
    return proxy_pattern


def check_domain_name(v: str) -> str:
    if v is not None:
        if len(v) > 255:
            raise ValueError('length shoud be less than 255')
        if not domain_name_regex.match(v):
            raise ValueError(
                "format error：Invalid domain name")
    return v


def check_elb_http_expected_codes(v: str) -> str:
    if v is not None:
        if len(v) > 64:
            raise ValueError('length shoud be less than 64')
        if not elb_http_expected_codes_regex.match(v):
            raise ValueError(
                "format error：Invalid httpExpectedCodes")
    return v


def check_description(description: str) -> str:
    if description is not None:
        if description.startswith('http:') or description.startswith('https:'):
            raise ValueError("should not start with http: or https:")
        if len(description) > 128:
            raise ValueError("length should be  0 - 128")
        if not desc_regex.match(description):
            raise ValueError(
                "format error：only support en letter、chinese、digits, ~!@#$%^&*()_\\-+= <>?:\"{}|,.\/;'[\]·~！@#￥%……&*（） ——\-+={}|《》？：“”【】、；‘'，。、")
    return description


def check_port_range(value: str) -> str:
    if value is not None:
        value_ary = value.split(":")
        if len(value_ary) == 1:
            if not value_ary[0].isdigit():
                raise ValueError("is not a number")
            check_port(int(value_ary[0]))
        elif len(value_ary) == 2:
            for i in value_ary:
                if not i.isdigit():
                    raise ValueError("is not a number")
                check_port(int(i))
            
            if int(value_ary[0]) > int(value_ary[1]):
                raise ValueError(
                    "port_start must be less than or equal to port_end")
        else:
            raise ValueError("invalid port range")
    return value


def check_port(port: int) -> int:
    if not 1 <= port <= 65535:
        raise ValueError("port number is 1 - 65535")
    return port


def check_target_group_algorithm(algorithm: str) -> str | None:
    if algorithm is not None and algorithm not in ('rr', 'wrr', 'lc', 'sh'):
        raise ValueError('only support rr / wrr / lc / sh')
    return algorithm


def check_sticky_session_mode(mode: str) -> str:
    # elb server not support source_ip
    if mode and mode.lower() not in ('close', 'insert', 'rewrite'):
        raise ValueError('only support CLOSE / INSERT / REWRITE')
    return mode


def check_cookie_expire(v: int | None, values):
    if 'session_sticky_mode' in values and values['session_sticky_mode'] == 'INSERT' and not v:
        raise ValueError('when sessionStictyMode is INSERT，field required')
    return v


def check_source_ip_timeout(v: int | None, values):
    if 'session_sticky_mode' in values and values['session_sticky_mode'] == 'SOURCE_IP' and not v:
        raise ValueError('when sessionStictyMode is SOURCE_IP，field required')
    return v


def check_rewrite_cookie_name(v: int | None, values):
    if 'session_sticky_mode' in values and values['session_sticky_mode'] == 'REWRITE' and not v:
        raise ValueError('when sessionStictyMode is REWRITE，field required')
    return v


def check_record_type_and_value(record_type, v):
    record_type = record_type.upper()
    if record_type == 'CNAME':
        if len(v) > 1:
            raise ValueError("when type is CNAME, only one value is supported")
        check_dns_name(v[0])
    elif record_type == 'A':
        for i in v:
            try:
                a = ipaddress.ip_address(i)
                if a.version != 4:
                    raise ValueError("invalid ip address")
            except ValueError:
                raise ValueError("invalid ip address")
    elif record_type == 'AAAA':
        for i in v:
            check_ipv6(i)
    elif record_type == 'MX':
        for i in v:
            check_mx(i)
    elif record_type == 'TXT':
        data = []
        for i in v:
            check_txt(i)
            data.append(f"\"{i}\"")
        return data
    return v


def check_ip(ip: str) -> str:
    if ip is not None:
        try:
            ipaddress.ip_address(ip)
        except ValueError:
            raise ValueError("invalid ip address")
    return ip


def check_mac(mac: str) -> str:
    if mac is not None:
        if not mac_regex.match(mac):
            raise ValueError("invalid MAC address")
    return mac


def check_mx(v: str) -> str:
    if not mx_value_regex.match(v):
        raise ValueError(f"zone record MX type value [{v}] was wrong format.")
    
    ary = v.split(" ")
    if len(ary) != 2:
        raise ValueError(f'zone record MX type value [{v}] was wrong format')
    priority, domain_name = ary
    if not priority.isdigit():
        raise ValueError('zone record MX type value was wrong format')
    
    if not 0 <= int(priority) <= 65535:
        raise ValueError(
            'zone record MX type value priority must be [0, 65535] range')
    
    check_dns_name(domain_name)
    return v


def check_txt(v: str) -> str:
    if not txt_value_regex.match(f"\"{v}\""):
        raise ValueError(f"zone record TXT type value [{v}] was wrong format")
    if len(v) > 4096:
        raise ValueError(
            f'zone record TXT type value [{v}] length should not greater than 4096')
    
    return v


def check_ipv6(ip: str) -> str:
    if ip is not None:
        try:
            a = ipaddress.ip_address(ip)
            if a.version != 6:
                raise ValueError("invalid ipv6 address")
        except ValueError:
            raise ValueError("invalid ipv6 address")
    return ip


def check_ipv6s(data: list[str]) -> list[str]:
    if not data:
        return []
    rst = set()
    for i in data:
        rst.add(check_ipv6(i))
    return list(rst)


def check_ips(data: list[str]) -> list[str]:
    if not data:
        return []
    
    rst = set()
    for i in data:
        rst.add(check_ip(i))
    return list(rst)


def check_subnet_type(subnet_type: str) -> str:
    if subnet_type not in ('common', 'cbm'):
        raise ValueError('only support common / cbm')
    return subnet_type


def check_cidr(cidr: str) -> str:
    if cidr is not None:
        try:
            ipaddress.ip_network(cidr)
        except ValueError:
            raise
    return cidr


def check_vpc_cidr(cidr: str) -> str:
    if cidr is not None:
        try:
            a = ipaddress.ip_network(cidr)
        except ValueError:
            raise
        
        if a.prefixlen > 28:
            raise ValueError('cidr prefix should be less than 28')
    return cidr


def check_cidrs(cidrs: List[str]) -> List[str]:
    if not cidrs:
        return []
    data = set()
    for cidr in cidrs:
        data.add(check_cidr(cidr))
    return list(data)


def check_mirror_filter_rule_port(port: str) -> str:
    if port is not None:
        if port == '-':
            return port
        else:
            ary = port.split('/')
            if len(ary) != 2:
                raise ValueError('format error')
            if not ary[0].isdigit() or not 1 <= int(ary[0]) <= 65535:
                raise ValueError('format error')
            
            if not ary[1].isdigit() or not 1 <= int(ary[1]) <= 65535:
                raise ValueError('format error')
    return port


def check_target_instance_type(v: str) -> str:
    if v.lower() not in ('vm', 'bm', 'ip'):
        raise ValueError('only support VM / BM / IP')
    return v


def check_private_nat_spec(spec: str) -> str:
    if spec not in ('small', 'medium', 'large', 'xlarge'):
        raise ValueError('spec must be one of small/medium/large/xlarge')
    return spec


def check_cycle_count(v, values, **kwargs):
    if 'cycle_type' not in values:
        return v
    
    cycle_type = values['cycle_type'].lower()
    match cycle_type:
        case 'on_demand':
            return v
        case 'year':
            if not v:
                raise ValueError(
                    'when cycleType is YEAR, cycleCount field required')
            if not 1 <= v <= 3:
                raise ValueError(
                    'when cycleType is YEAR, cycleCount should be range  1 - 3')
        case 'month':
            if not v:
                raise ValueError(
                    'when cycleType is MONTH, cycleCount field required')
            if not 1 <= v <= 11:
                raise ValueError(
                    'when cycleType is MONTH, cycleCount should be range  1 - 11')
        case _:
            return v


def check_ipv4_map_ipv6_cidr(v):
    if v.startswith("::ffff:") or v.startswith("0:0:0:0:0:ffff:"):
        ipv6_network = ipaddress.IPv6Network(v)
        ipv6_address = ipaddress.IPv6Address(ipv6_network.network_address.packed)
        if ipv6_address.ipv4_mapped:
            raise ValueError("ipv4 map 1pv6 is not allowed")
    return v


def check_ipv4_map_ipv6_addr(v):
    try:
        if v.startswith("::ffff:") or v.startswith("0:0:0:0:0:ffff:"):
            ipv6 = ipaddress.IPv6Address(v)
            if ipv6.ipv4_mapped:
                raise ValueError("ipv4 map 1pv6 is not allowed")
    except ipaddress.AddressValueError:
        pass
    return v



from typing import List

from pydantic import BaseModel, Field

from .base import BaseResponse


class Item(BaseModel):

    name: str
    desc: str
    unit: str



class MonitorItem(BaseModel):

    device_type: str = Field(alias="deviceType")
    items: List[Item]


class MonitorItemReturnObj(BaseModel):
    monitor_items: List[MonitorItem] = Field(alias='monitorItems')


class MonitorItemResponse(BaseResponse):

    return_obj: MonitorItemReturnObj = Field(alias='returnObj')



from typing import Optional, List

from pydantic import BaseModel, Field

from .base import BaseResponse

__all__ = [
    'CreateSubnetResponse',
    "ListSubnetResponse",
    "NewListSubnetResponse",
    "ShowSubnetResponse",
    "CheckIpAvailableResponse"
]


class CreateSubnet(BaseModel):
    vpc_id: str = Field(alias='vpcID')


class CreateSubnetResponse(BaseResponse):
    return_obj: Optional[CreateSubnet] = Field(None, alias='returnObj')


class Subnet(BaseModel):
    subnet_id: str = Field(alias="subnetID")
    name: str
    description: str
    vpc_id: str = Field(alias="vpcID")
    availability_zones: List[str] = Field(alias="availabilityZones")
    route_table_id: str = Field(alias="routeTableID")
    acl_id: str = Field(alias="networkAclID")
    cidr: str = Field(alias="CIDR")
    gateway_ip: str = Field(alias="gatewayIP")
    start: str
    end: str
    available_ip_count: int = Field(alias="availableIPCount")
    ipv6_enabled: int = Field(alias="ipv6Enabled")
    enable_ipv6: bool = Field(alias="enableIpv6")
    ipv6_cidr: str = Field(alias="ipv6CIDR")
    ipv6_start: str = Field(alias="ipv6Start")
    ipv6_end: str = Field(alias="ipv6End")
    ipv6_gateway_ip: str = Field(alias="ipv6GatewayIP")
    dns_list: List[str] = Field(alias="dnsList")
    ntp_list: List[str] = Field(alias="ntpList")
    type: int
    created_at: str = Field(alias="createdAt")
    updated_at: str = Field(alias="updatedAt")


class ListSubnetResponse(BaseResponse):
    return_obj: Optional[List[Subnet]] = Field(None, alias='returnObj')
    total_count: int = Field(alias='totalCount')
    current_count: int = Field(alias='currentCount')
    total_page: int = Field(alias='totalPage')


class ListSubnets(BaseModel):
    subnets: List[Subnet]
    total_count: int = Field(alias='totalCount')
    current_count: int = Field(alias='currentCount')
    total_page: int = Field(alias='totalPage')


class NewListSubnetResponse(BaseResponse):
    return_obj: ListSubnets = Field(alias='returnObj')


class ShowSubnetResponse(BaseResponse):
    return_obj: Subnet = Field(None, alias='returnObj')


class IpAvailable(BaseModel):
    is_available: bool = Field(alias="isAvailable")


class CheckIpAvailableResponse(BaseResponse):
    return_obj: IpAvailable = Field(None, alias='returnObj')



from typing import Optional

from pydantic import Field

from pyctyun.rules.rule import BaseParamModel


class MonitorItemParam(BaseParamModel):

    region_id: str = Field(alias='regionID', min_length=1)
    service: Optional[str] = Field(None)
    item_type: Optional[str] = Field(None, alias='itemType')



from typing import Optional, List

from pydantic import BaseModel, Field

from .base import BaseResponse

__all__ = [
    'CreateHavipResponse',
    "ListHavipResponse",
    "ShowHavipResponse"
]


class CreateHavip(BaseModel):
    havip_id: str = Field(alias='uuid')
    ipv4: str
    ipv6: str


class NetworkInfo(BaseModel):
    
    eip_id: str = Field(alias="eipID")


class BindPorts(BaseModel):
    port_id: str = Field(alias="portID")
    role: str
    created_at: str = Field(alias="createdAt")


class InstanceInfo(BaseModel):
    
    instance_name: str = Field(alias="instanceName")
    id: str
    private_ip: str = Field(alias="privateIp")
    private_ipv6: str = Field(alias="privateIpv6")
    public_ip: str = Field(alias="publicIp")


class Havip(BaseModel):

    id: str
    ipv4: str
    vpc_id: str = Field(alias="vpcID")
    subnet_id: str = Field(alias="subnetID")
    instance_info: List[InstanceInfo] = Field(alias="instanceInfo")
    bind_ports: List[BindPorts] = Field([], alias="bindPorts")
    network_info: List[NetworkInfo] = Field([], alias="networkInfo")


class CreateHavipResponse(BaseResponse):
    return_obj: Optional[CreateHavip] = Field(None, alias='returnObj')
    
    
class ListHavipResponse(BaseResponse):
    return_obj: List[Havip] = Field(None, alias='returnObj')
    
    
class ShowHavipResponse(BaseResponse):
    return_obj: Havip = Field(None, alias='returnObj')


from typing import List

from pydantic import BaseModel, Field

from .base import BaseResponse


class Item(BaseModel):
    item_name: str = Field(alias='itemName')
    value: str
    sampling_time: str = Field(alias='samplingTime')


class MonitorItem(BaseModel):
    region_id: str = Field(alias="regionID")
    device_uuid: str = Field(alias='deviceUUID')
    item_list: List[Item] = Field(alias='itemList')


class RealtimeMetricItem(BaseModel):
    result: List[MonitorItem]


class RealtimeMetricResponse(BaseResponse):
    return_obj: RealtimeMetricItem = Field(alias='returnObj')



from typing import Optional

from pydantic import BaseModel, Field

from .base import BaseResponse


class DhcpOption(BaseModel):
    dhcp_option_sets_id: str = Field(alias='dhcpOptionSetsID')
    name: str
    description: str
    domain_name: list[str] = Field(alias="domainName")
    dns_list: list[str] = Field(alias="dnsList")
    vpc_list: list[str] = Field(alias="vpcList")
    created_at: str = Field(alias='createdAt')
    updated_at: str = Field(alias='updatedAt')


class ShowDhcpResponse(BaseResponse):
    return_obj: Optional[DhcpOption] = Field(alias='returnObj')


class ListDhcp(BaseModel):
    results: list[DhcpOption]
    total_count: int = Field(alias='totalCount')
    current_count: int = Field(alias='currentCount')
    total_page: int = Field(alias='totalPage')


class ListDhcpResponse(BaseResponse):
    return_obj: Optional[ListDhcp] = Field(alias='returnObj')


class UnbindVpc(BaseModel):
    id: str
    name: str


class UnbindVpcResponse(BaseResponse):
    return_obj: Optional[list[UnbindVpc]] = Field(alias='returnObj')


class Vpc(BaseModel):
    vpc_id: str = Field(alias='vpcID')
    name: str
    cidr: str
    status: str
    created_at: str = Field(alias='createdAt')
    secondary_cidrs: list[str] = Field(alias='secondaryCidrs')


class BindVpc(BaseModel):
    results: list[Vpc]
    total_count: int = Field(alias='totalCount')
    current_count: int = Field(alias='currentCount')
    total_page: int = Field(alias='totalPage')


class BindVpcResponse(BaseResponse):
    return_obj: Optional[BindVpc] = Field(alias='returnObj')
    
    
class DhcpOptionSetID(BaseModel):
    
    dhcp_option_sets_id: str = Field(alias='dhcpOptionSetsID')
    
    
class DhcpOptionSetIDResponse(BaseResponse):
    return_obj: Optional[DhcpOptionSetID] = Field(alias='returnObj')




import os
import unittest

from pyctyun.apis.monitor.monitor import MonitorApi
from pyctyun.param.monitor import RealtimeMonitorParam


class TestMonitorApi(unittest.TestCase):
    client = None
    vpc_id = None
    
    @classmethod
    def setUpClass(cls) -> None:
        cls.client = MonitorApi(ak=os.getenv("access_key"),
                                sk=os.getenv("secret_key"),
                                endpoint=os.getenv("endpoint"))
    
    def test_get_realtime_metric(self):
        param = RealtimeMonitorParam(**{
            'regionID': 'bb9fdb42056f11eda1610242ac110002',
            'deviceUUIDList': ['121.225.97.89']})
        res = self.client.get_realtime_metric(param)
        print(res)



import os
import unittest

from pyctyun.apis.network.eip import EipApi
from pyctyun.param.eip import ListEipParam


class TestEipApi(unittest.TestCase):
    client = None
    vpc_id = None
    
    @classmethod
    def setUpClass(cls) -> None:
        cls.client = EipApi(ak=os.getenv("access_key"),
                            sk=os.getenv("secret_key"),
                            endpoint=os.getenv("endpoint"))
    
    def test_list_eip(self):
        resp = self.client.list_eip(ListEipParam(**{
            'regionID': 'bb9fdb42056f11eda1610242ac110002',
            'clientToken': 'xxx',
            "pageNo": 1,
            "pageSize": 1
        }))
        self.assertEqual(800, resp.status_code)
        data = resp.return_obj.model_dump(by_alias=True)
        self.assertIn('eips', data)
        self.assertEqual(1, resp.current_count)
        
        for eip in data['eips']:
            self.assertIn('ID', eip)
            self.assertIn('name', eip)
            self.assertIn('eipAddress', eip)
            self.assertIn('associationID', eip)
            self.assertIn('associationType', eip)
            self.assertIn('privateIpAddress', eip)
            self.assertIn('bandwidth', eip)
            self.assertIn('bandwidthID', eip)
            self.assertIn('bandwidthType', eip)
            self.assertIn('status', eip)
            self.assertIn('tags', eip)
            self.assertIn("createdAt", eip)
            self.assertIn("updatedAt", eip)
            self.assertIn("expiredAt", eip)
    
    def test_new_list_eip(self):
        resp = self.client.new_list_eip(ListEipParam(**{
            'regionID': 'bb9fdb42056f11eda1610242ac110002',
            'clientToken': 'xxx',
            'pageNumber': 2,
            'pageSize': 2
        }))
        self.assertEqual(800, resp.status_code)
        data = resp.return_obj.model_dump(by_alias=True)
        # print(data)
        self.assertIn('eips', data)
        self.assertIn("currentCount", data)
        self.assertIn("totalPage", data)
        self.assertIn("totalCount", data)
        self.assertEqual(2, resp.return_obj.current_count)

        for eip in data['eips']:
            self.assertIn('ID', eip)
            self.assertIn('name', eip)
            self.assertIn('eipAddress', eip)
            self.assertIn('associationID', eip)
            self.assertIn('associationType', eip)
            self.assertIn('privateIpAddress', eip)
            self.assertIn('bandwidth', eip)
            self.assertIn('bandwidthID', eip)
            self.assertIn('bandwidthType', eip)
            self.assertIn('status', eip)
            self.assertIn('tags', eip)
            self.assertIn("createdAt", eip)
            self.assertIn("updatedAt", eip)
            self.assertIn("expiredAt", eip)



import base64
import datetime
import hashlib
import hmac
import json
import uuid

from urllib.parse import quote


class Auth(object):
    """Get access token by crop id and secret."""

    def __init__(self, ak: str, sk: str):
        """Init.
        :param str app_key: App key
        :param str app_secret: App secret
        :param str sign_method:
        :param str endpoint:
        """
        self.ak = ak
        self.sk = sk

    def hmac_sha256(self, secret, data):
        if isinstance(secret, str):
            secret = secret.encode('utf8')
        if isinstance(data, str):
            data = data.encode('utf8')
        return hmac.new(secret, data, digestmod=hashlib.sha256).digest()

    def base64_of_hmac(self, data):
        return base64.b64encode(data).decode('utf8')

    def get_request_uuid(self):
        return str(uuid.uuid1())

    def get_sorted_str(self, data):
        sorted_data = sorted(data.items(), key=lambda item: item[0])
        # str_list = map(lambda (x, y): '%s=%s' % (x, y), sorted_data)
        str_list = map(lambda x: '%s=%s' %
                       (x[0], quote(str(x[1]))), sorted_data)
        return '&'.join(str_list)

    def build_sign(self, query_params, body_params, eop_date, request_uuid):
        body_str = json.dumps(body_params)
        body_digest = hashlib.sha256(body_str.encode('utf-8')).hexdigest()
        # 请求头中必要的两个参数
        header_str = 'ctyun-eop-request-id:%s\neop-date:%s\n' % (
            request_uuid, eop_date)
        # url中的参数，或get参数
        query_str = self.get_sorted_str(query_params)

        signature_str = '%s\n%s\n%s' % (header_str, query_str, body_digest)
        # sys.stdout.write(repr('signature_str is: %s' % signature_str))
        sign_date = eop_date.split('T')[0]

        # 计算鉴权密钥
        k_time = self.hmac_sha256(self.sk, eop_date)
        k_ak = self.hmac_sha256(k_time, self.ak)
        k_date = self.hmac_sha256(k_ak, sign_date)

        signature_base64 = self.base64_of_hmac(
            self.hmac_sha256(k_date, signature_str))
        # 构建请求头的鉴权字段值
        sign_header = '%s Headers=ctyun-eop-request-id;eop-date Signature=%s' % (
            self.ak, signature_base64)
        return sign_header

    def generate_sign_headers(self,
                              query_params: dict = None,
                              body_params: dict = None) -> dict:
        query_params = query_params or {}
        body_params = body_params or {}

        now = datetime.datetime.now()
        eop_date = now.strftime("%Y%m%dT%H%M%SZ")
        request_uuid = self.get_request_uuid()
        authorization = self.build_sign(query_params=query_params,
                                        body_params=body_params,
                                        eop_date=eop_date,
                                        request_uuid=request_uuid)
        headers = {
            'eop-date': eop_date,
            'ctyun-eop-request-id': request_uuid,
            'Eop-Authorization': authorization,
        }
        return headers



from typing import Optional

from pydantic import Field, field_validator, StrictBool, StrictStr

from pyctyun.rules.rule import check_name, check_description, check_cidr, check_ips, check_ip, check_subnet_type, \
    BaseParamModel, PaginationGetParamModel


class CreateSubnetParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)
    name: StrictStr = Field(max_length=31, min_length=1)
    description: StrictStr = Field('', min_length=0, max_length=128)
    cidr: StrictStr = Field(alias='CIDR', min_length=1)
    enable_ipv6: Optional[StrictBool] = Field(alias='enableIpv6')
    dns_list: Optional[list[str]] = Field(alias='dnsList', max_items=4)
    subnet_gateway_ip: Optional[str] = Field(alias='subnetGatewayIP')
    subnet_type: StrictStr = Field("common", alias='subnetType')
    project_id: Optional[StrictStr] = Field('0', alias='projectID')
    
    _check_description = field_validator('description')(check_description)
    _check_name = field_validator('name')(check_name)
    _check_cidr = field_validator('cidr')(check_cidr)
    _check_dns_list = field_validator('dns_list')(check_ips)
    _check_subnet_gateway_ip = field_validator('subnet_gateway_ip')(check_ip)
    _check_subnet_type = field_validator('subnet_type')(check_subnet_type)


class DeleteSubnetParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)


class UpdateSubnetParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)
    name: Optional[StrictStr]
    dns_list: Optional[list[str]] = Field(alias='dnsList', max_items=4)
    description: Optional[StrictStr]
    
    _check_description = field_validator('description')(check_description)
    _check_name = field_validator('name')(check_name)
    _check_dns_list = field_validator('dns_list')(check_ips)


class SubnetReplaceRouteTableParam(BaseParamModel):
    client_token: Optional[StrictStr] = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)
    route_table_id: StrictStr = Field(alias='routeTableID', min_length=1)


class SubnetDisassociateRouteTableParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)


class SubnetCreateACLParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)
    project_id: StrictStr = Field('0', alias='projectID')
    name: StrictStr = Field(min_length=1)
    description: Optional[StrictStr]
    
    _check_name = field_validator('name')(check_name)
    _check_description = field_validator('description')(check_description)


class SubnetEnableIPv6Param(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)


class SubnetDisassociateACLParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)
    acl_id: StrictStr = Field(alias="aclID", min_length=1)


class SubnetReplaceACLParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)
    acl_id: StrictStr = Field(alias="aclID", min_length=1)


class UpdateSubnetIPv6StatusParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)
    enable_ipv6: StrictBool = Field(alias="enableIpv6")


class ListSubnetsParam(PaginationGetParamModel):
    region_id: Optional[str] = Field(alias='regionID')
    subnet_id: Optional[str] = Field(alias='subnetID')
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)


class ShowSubnetParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)


class CheckIpAvaliableParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)
    ip: StrictStr = Field(alias='fixedIP', min_length=1)
    
    _check_ip = field_validator('ip')(check_ip)



from typing import List, Optional

from pydantic import BaseModel, Field

from .base import BaseResponse


class Eip(BaseModel):
    id: str = Field(alias="ID")
    name: str
    eip_address: str = Field(alias="eipAddress")
    association_id: str = Field("", alias="associationID")
    association_type: Optional[str] = Field(None, alias="associationType")
    private_ip_address: Optional[str] = Field(None, alias="privateIpAddress")
    bandwidth: int
    bandwidth_id: str = Field("", alias="bandwidthID")
    bandwidth_type: str = Field(alias="bandwidthType")
    status: str
    tags: str
    created_at: str = Field(alias="createdAt")
    updated_at: str = Field(alias="updatedAt")
    expired_at: Optional[str] = Field(None, alias="expiredAt")


class ListEip(BaseModel):
    eips: List[Eip]


class NewListEip(BaseModel):
    eips: List[Eip]
    total_count: int = Field(alias='totalCount')
    current_count: int = Field(alias='currentCount')
    total_page: int = Field(alias='totalPage')


class NewListEipResponse(BaseResponse):
    return_obj: NewListEip = Field(alias='returnObj')


class ListEipResponse(BaseResponse):
    return_obj: ListEip = Field(alias='returnObj')
    total_count: int = Field(alias='totalCount')
    current_count: int = Field(alias='currentCount')
    total_page: int = Field(alias='totalPage')



from typing import Optional

from pydantic import Field, field_validator, StrictBool, StrictStr

from pyctyun.rules.rule import BaseParamModel, PaginationGetParamModel, check_name, check_description, \
    check_vpc_cidr


class CreateVPCParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    name: StrictStr
    description: Optional[StrictStr] = None
    cidr: StrictStr = Field(alias='CIDR', min_length=1)
    enable_ipv6: Optional[StrictBool] = Field(False, alias='enableIpv6')
    ipv6_segment_pool_id: Optional[StrictStr] = Field(None, alias='ipv6SegmentPoolID')
    
    _check_name = field_validator('name')(check_name)
    _check_description = field_validator('description')(check_description)
    _check_cidr = field_validator('cidr')(check_vpc_cidr)


class DeleteVPCParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)


class ShowVPCParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)
    project_id: StrictStr = Field('0', alias='projectID')


class ListVPCParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vpc_id: Optional[StrictStr] = Field(None, alias='vpcID')
    vpc_name: Optional[StrictStr] = Field(None, alias='vpcName')
    project_id: StrictStr = Field('0', alias='projectID')


class UpdateVPCIPv6StatusParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)
    enable_ipv6: StrictBool = Field(alias='enableIpv6')


class UpdateVPCParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)
    name: Optional[StrictStr] = Field(max_length=31, min_length=2)
    description: Optional[StrictStr] = None
    
    _check_name = field_validator('name')(check_name)
    _check_description = field_validator('description')(check_description)



import os
import unittest

from pyctyun.apis.monitor.monitor_item import MonitorItemApi
from pyctyun.param.montior_item import MonitorItemParam


class TestMonitorItemApi(unittest.TestCase):
    client = None
    vpc_id = None
    
    @classmethod
    def setUpClass(cls) -> None:
        cls.client = MonitorItemApi(ak=os.getenv("access_key"),
                                    sk=os.getenv("secret_key"),
                                    endpoint=os.getenv("endpoint"))
    
    def test_get_monitor_items(self):
        param = MonitorItemParam(**{'regionID': 'bb9fdb42056f11eda1610242ac110002'})
        res = self.client.get_monitor_items(param)
        print(res)



import requests

from . import exceptions


class Transport(object):
    """Implement transport between api servers and clinet."""

    def __init__(self, endpoint):
        """Init.
        :param str endpoint: Api endpoint
        """
        self.endpoint = endpoint.rstrip('/')

    def perform_request(self, api, params: dict, method='GET', headers=None, timeout=1):
        url = f'{self.endpoint}{api}'

        try:
            if method == 'POST':
                resp = requests.post(url, json=params, headers=headers)
            else:
                resp = requests.get(url, params=params, headers=headers, json={})
            return resp.json()
        except (requests.exceptions.ConnectTimeout, requests.exceptions.ReadTimeout) as e:
            raise exceptions.TimeoutError(
                status_code=504, reason=str(e))



from typing import List, Optional

from pydantic import BaseModel, Field

from .base import BaseResponse

__all__ = [
    'CreateVpcResponse',
    'ListVpcResponse',
    'ShowVpcResponse'
]


class CreateVpc(BaseModel):
    vpc_id: str = Field(alias='vpcID')


class Vpc(BaseModel):
    vpc_id: str = Field(alias='vpcID')
    name: str
    description: str
    cidr: str = Field(alias='CIDR')
    ipv6_enabled: bool = Field(alias='ipv6Enabled')
    enable_ipv6: bool = Field(alias='enableIpv6')
    ipv6_cidrs: List[str] = Field(alias='ipv6CIDRS')
    subnet_ids: List[str] = Field(alias='subnetIDs')
    nat_gateway_ids: List[str] = Field(alias='natGatewayIDs')
    secondary_cidrs: List[str] = Field(alias='secondaryCIDRS')
    project_id: str = Field(alias='projectID')


class ListVpc(BaseModel):
    vpcs: List[Vpc]
    total_count: int = Field(alias='totalCount')
    current_count: int = Field(alias='currentCount')
    total_page: int = Field(alias='totalPage')


class CreateVpcResponse(BaseResponse):
    return_obj: Optional[CreateVpc] = Field(None, alias='returnObj')


class ListVpcResponse(BaseResponse):
    return_obj: Optional[ListVpc] = Field(None, alias='returnObj')


class ShowVpcResponse(BaseResponse):
    return_obj: Optional[Vpc] = Field(None, alias='returnObj')



from pydantic import BaseModel, Field


class BaseResponse(BaseModel):

    status_code: int = Field(alias='statusCode')
    message: str
    description: str
    error_code: str = Field(alias='errorCode')



from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param import prefix_list
from pyctyun.response import prefix_list as prefix_list_response


class PrefixListApi(CtapiBaseClient):
    
    def create_prefix_list(self, param: prefix_list.CreatePrefixListParam) -> prefix_list_response.CreatePrefixListResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist/create', params=params, method='POST')
        return prefix_list_response.CreatePrefixListResponse(**data)
    
    def delete_prefix_list(self, param: prefix_list.DeletePrefixListParam) -> prefix_list_response.DeletePrefixListResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist/delete', params=params, method='POST')
        return prefix_list_response.DeletePrefixListResponse(**data)
    
    def list_prefix_list(self, param: prefix_list.QueryPrefixListParam) -> prefix_list_response.ListPrefixResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/prefixlist/query', params=params, method='GET')
        return prefix_list_response.ListPrefixResponse(**data)
    
    def show_prefix_list(self, param: prefix_list.ShowPrefixListParam) -> prefix_list_response.ShowPrefixListResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist/show', params=params, method='GET')
        return prefix_list_response.ShowPrefixListResponse(**data)
    
    def clone_prefix_list(self, param: prefix_list.ClonePrefixListParam) -> prefix_list_response.ClonePrefixListResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist/clone', params=params, method='POST')
        return prefix_list_response.ClonePrefixListResponse(**data)
    
    def update_prefix_list(self, param: prefix_list.UpdatePrefixListParam) -> prefix_list_response.ModifyPrefixListResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist/update', params=params, method='POST')
        return prefix_list_response.ModifyPrefixListResponse(**data)
    
    def create_prefix_list_rule(self, param: prefix_list.CreatePrefixListRuleParam) -> prefix_list_response.CreatePrefixListResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist_rule/create', params=params, method='POST')
        return prefix_list_response.CreatePrefixListResponse(**data)
    
    def delete_prefix_list_rule(self, param: prefix_list.DeletePrefixListRuleParam) -> prefix_list_response.DeletePrefixListRuleResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist_rule/delete', params=params, method='POST')
        return prefix_list_response.DeletePrefixListRuleResponse(**data)
    
    def modify_prefix_list_rule(self, param: prefix_list.UpdatePrefixListRuleParam) -> prefix_list_response.UpdatePrefixListRuleResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist_rule/update', params=params, method='POST')
        return prefix_list_response.UpdatePrefixListRuleResponse(**data)
    
    def list_bind_resource(self, param: prefix_list.ListBindResourceParam) -> prefix_list_response.ListResourceResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/prefixlist/get_associations', params=params, method='GET')
        return prefix_list_response.ListResourceResponse(**data)
        


from typing import List, Dict

from dataclasses import dataclass

from src.apis.base import CtapiBaseClient


@dataclass
class QueryDelegateRoleParam:
    user_id: str
    
    
@dataclass
class DelegateRoleResponse:
    
    account_id: str
    role_list: List[str]
    role_info: Dict


class DelegateApi(CtapiBaseClient):
    
    def query_delegate_role(self, param: QueryDelegateRoleParam, headers=None, timeout=1):
        params = {
            "assumeUserId": param.user_id,
        }
        data = self.perform_request('/v1/delegate/queryDelegateRoleValue', params, 'POST', headers=headers, timeout=timeout)
        if data['statusCode'] == '800':
            return_obj = data.get("returnObj") or {}
            account_id = return_obj.get("assumeAccountId")
            role_list = return_obj.get("roleList", [])
            role_code_list = []
            role_info = {}
            for role in role_list:
                role_code = role.get("roleCode")
                role_code_list.append(role_code)
                role_info[role_code] = [item["value"] for item in role.get("valueList", []) if 'value' in item]
    
            return DelegateRoleResponse(
                account_id=account_id,
                role_list=role_code_list,
                role_info=role_info
            )
        return DelegateRoleResponse(
            account_id='',
            role_list=[],
            role_info={}
        )



from typing import Optional

from pydantic import BaseModel, Field

from .base import BaseResponse


class PrefixListRule(BaseModel):
    prefix_list_rule_id: str = Field(alias='prefixListRuleID')
    cidr: str
    description: str


class PrefixList(BaseModel):
    prefix_list_id: str = Field(alias='prefixListID')
    name: str
    limit: int
    address_type: str = Field(alias='addressType')
    description: str
    created_at: str = Field(alias='createdAt')
    updated_at: str = Field(alias='updatedAt')
    prefix_list_rules: list[PrefixListRule] = Field(alias='prefixListRules')


class Resource(BaseModel):
    resource_id: str = Field(alias='resourceID')
    resource_name: str = Field(alias='resourceName')
    resource_type: str = Field(alias='resourceType')
    
    
class DeletePrefixListRule(BaseModel):
    
    prefix_list_id: str = Field(alias='prefixListID')
    prefix_list_rule_id: str = Field(alias='prefixListRuleID')


class PrefixListIDResponse(BaseModel):
    prefix_list_id: str = Field(alias='prefixListID')


class ModifyPrefixListResponse(BaseResponse):
    return_obj: Optional[PrefixListIDResponse] = Field(alias='returnObj')


class ClonePrefixListResponse(BaseResponse):
    return_obj: Optional[PrefixListIDResponse] = Field(alias='returnObj')


class CreatePrefixListResponse(BaseResponse):
    return_obj: Optional[PrefixListIDResponse] = Field(alias='returnObj')


class DeletePrefixListResponse(BaseResponse):
    return_obj: Optional[PrefixListIDResponse] = Field(alias='returnObj')


class ListPrefixList(BaseModel):
    results: list[PrefixList]
    total_page: int = Field(alias='totalPage')
    current_count: int = Field(alias='currentCount')
    total_count: int = Field(alias='totalCount')


class ListPrefixResponse(BaseResponse):
    return_obj: Optional[ListPrefixList] = Field(alias='returnObj')


class ShowPrefixListResponse(BaseResponse):
    return_obj: Optional[PrefixList] = Field(alias='returnObj')


class ListResource(BaseModel):
    results: list[Resource]
    total_page: int = Field(alias='totalPage')
    current_count: int = Field(alias='currentCount')
    total_count: int = Field(alias='totalCount')


class ListResourceResponse(BaseResponse):
    return_obj: Optional[ListResource] = Field(alias='returnObj')
    
    
class DeletePrefixListRuleResponse(BaseResponse):
    return_obj: Optional[DeletePrefixListRule] = Field(alias='returnObj')


class UpdatePrefixListRuleResponse(BaseResponse):
    return_obj: Optional[DeletePrefixListRule] = Field(alias='returnObj')


from typing import Optional

from pydantic import Field, field_validator, StrictStr

from pyctyun.rules.rule import check_ip, check_description, BaseParamModel


class CreateVipParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    network_id: Optional[StrictStr] = Field(alias='networkID')
    subnet_id: StrictStr = Field(alias='subnetID', min_length=1)
    ip_address: Optional[StrictStr] = Field(alias='ipAddress')
    vip_type: Optional[StrictStr] = Field('v4', alias='vipType')
    description: Optional[StrictStr] = Field(alias='description')
    
    _check_ip = field_validator('ip_address')(check_ip)
    _check_description = field_validator('description')(check_description)
    
    @field_validator('vip_type')
    def check_vip_type(cls, value: str):
        if value is None:
            return 'v4'
        if value is not None and value not in ('v4', 'v6'):
            raise ValueError('only support v4 / v6')
        return value


class DeleteVipParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    vip_id: StrictStr = Field(alias='haVipID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)


class ShowVipParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vip_id: StrictStr = Field(alias='haVipID', min_length=1)


class ListVipFilter(BaseParamModel):
    key: StrictStr
    value: StrictStr
    
    @field_validator('key', mode='after')
    def check_key(cls, v):
        if v not in ('haVipID', 'vpcID', 'subnetID'):
            raise ValueError('only support haVipID / vpcID / subnetID')
        return v


class ListVipParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    project_id: Optional[StrictStr] = Field('0', alias='projectID')
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    filters: Optional[list[ListVipFilter]] = Field(None)


class BindOrUnbindVipParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    resource_type: StrictStr = Field(alias='resourceType', min_length=1)
    ha_vip_id: StrictStr = Field(alias='haVipID', min_length=1)
    instance_id: Optional[StrictStr] = Field(None, alias='instanceID')
    network_interface_id: Optional[StrictStr] = Field(None, alias='networkInterfaceID')
    floating_id: Optional[StrictStr] = Field(None, alias='floatingID')
    project_id: StrictStr = Field('0', alias='projectID')
    
    @field_validator('resource_type', mode='after')
    def check_resource_type(cls, value: str):
        tmp = value.lower()
        if tmp not in ('vm', 'pm', 'network'):
            raise ValueError('only support VM / PM / NETWORK')
        return tmp
    
    @field_validator('network_interface_id', mode='after')
    def check_network_interface_id(cls, v, values):
        if 'resource_type' not in values.data:
            return v
        if values.data['resource_type'].lower() in ('vm', 'pm') and not v:
            raise ValueError("when resourceType is vm / pm, networkInterfaceID field required")
        return v
    
    @field_validator('instance_id', mode='after')
    def check_instance_id(cls, v, values):
        if 'resource_type' not in values.data:
            return v
        
        if values.data['resource_type'].lower() in ('vm', 'pm') and not v:
            raise ValueError("when resourceType is vm / pm, instanceID field required")
        return v
    
    @field_validator('floating_id', mode='after')
    def check_floating_id(cls, v, values):
        if 'resource_type' not in values.data:
            return v
        
        if values.data['resource_type'].lower() in ('network',) and not v:
            raise ValueError("when resourceType is NETWORK, floating_id field required")
        return v


from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param import havip
from pyctyun.response.base import BaseResponse
from pyctyun.response.havip import CreateHavipResponse, ListHavipResponse, ShowHavipResponse


class HavipApi(CtapiBaseClient):
    
    def create_havip(self, param: havip.CreateVipParam) -> CreateHavipResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/havip/create', params=params, method='POST')
        return CreateHavipResponse(**data)
    
    def delete_havip(self, param: havip.DeleteVipParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/havip/delete', params=params, method='POST')
        return BaseResponse(**data)
    
    def show_havip(self, param: havip.ShowVipParam) -> ShowHavipResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/havip/show', params=params, method='GET')
        return ShowHavipResponse(**data)
    
    def list_havip(self, param: havip.ListVipParam) -> ListHavipResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/havip/list', params=params, method='POST')
        return ListHavipResponse(**data)
    
    def bind_havip(self, param: havip.BindOrUnbindVipParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/havip/bind', params=params, method='POST')
        return BaseResponse(**data)
    
    def unbind_havip(self, param: havip.BindOrUnbindVipParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/havip/unbind', params=params, method='POST')
        return BaseResponse(**data)



from typing import List

from pydantic import Field

from pyctyun.rules.rule import BaseParamModel


class RealtimeMonitorParam(BaseParamModel):
    region_id: str = Field(alias='regionID', min_length=1)
    device_uud_list: List[str] = Field(alias='deviceUUIDList', min_items=1)



from typing import List, Optional

from pydantic import conint, Field, StrictStr, field_validator

from pyctyun.rules.rule import BaseParamModel, check_description, \
    check_record_type, check_record_type_and_value, PaginationGetParamModel


class CreateZoneRecordParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)
    name: Optional[StrictStr]
    description: Optional[StrictStr]
    record_type: StrictStr = Field(alias="type", min_length=1)
    ttl: Optional[conint(strict=True, ge=300, le=2147483647)] = Field(default=300, alias='TTL')
    value_list: List[StrictStr] = Field(alias="valueList", min_items=1, max_items=8, min_length=1)
    
    _check_record_type = field_validator('record_type')(check_record_type)
    _check_description = field_validator('description')(check_description)
    
    @field_validator('value_list')
    def check_value_list(cls, v, values):
        if 'record_type' not in values:
            return v
        record_type = values['record_type']
        return check_record_type_and_value(record_type, v)


class DeleteZoneRecordParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    zone_record_id: StrictStr = Field(alias='zoneRecordID', min_length=1)


class ShowZoneRecordParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_record_id: StrictStr = Field(alias='zoneRecordID', min_length=1)


class ModifyZoneRecordParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: Optional[StrictStr] = Field(alias='clientToken', min_length=1, max_length=64)
    zone_record_id: StrictStr = Field(alias='zoneRecordID', min_length=1)
    description: Optional[StrictStr]
    ttl: Optional[conint(strict=True, ge=300, le=2147483647)] = Field(alias='TTL')
    value_list: List[StrictStr] = Field(alias="valueList", min_items=1, min_length=1)
    
    _check_description = field_validator('description')(check_description)


class ModifyZoneRecordDescParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: Optional[StrictStr] = Field(alias='clientToken', min_length=1, max_length=64)
    zone_record_id: StrictStr = Field(alias='zoneRecordID', min_length=1)
    description: Optional[StrictStr]
    
    _check_description = field_validator('description')(check_description)


class BatchDeleteZoneRecordParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    zone_record_ids: list[StrictStr] = Field(alias='zoneRecordIDs', min_items=1, max_items=10, min_length=1)


class ListSharedZoneRecordParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)



from typing import List, Optional

from pydantic import Field, StrictBool, StrictStr, StrictInt, conint, field_validator

from pyctyun.rules.rule import check_description, BaseParamModel, check_name, PaginationGetParamModel, check_uuid, \
    check_uuids



class CreateSecurityGroupParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    project_id: Optional[StrictStr] = Field('0', alias='projectID')
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)
    name: StrictStr
    description: Optional[StrictStr]
    
    _check_description = field_validator('description')(check_description)
    _check_name = field_validator('name')(check_name)


class RemoveSecurityGroupParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    az_name: StrictStr = Field('', alias='azName')
    project_id: StrictStr = Field('0', alias='projectID')
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)


class UnbindSecurityGroupParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    instance_id: StrictStr = Field(alias='instanceID', min_length=1)
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)


class BindSecurityGroupParam(BaseParamModel):
    network_interface_id: StrictStr = Field('', alias='networkInterfaceID')
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    instance_id: StrictStr = Field(alias='instanceID', min_length=1)
    action: StrictStr
    
    @field_validator('action')
    def check_action(cls, value):
        if value != 'joinSecurityGroup':
            raise ValueError('action must be joinSecurityGroup')
        return value


class BatchBindSecurityGroupParam(BaseParamModel):
    network_interface_id: Optional[StrictStr] = Field(alias='networkInterfaceID')
    security_group_ids: list[str] = Field(alias='securityGroupIDs', min_items=1, max_items=10)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    instance_id: StrictStr = Field(alias='instanceID', min_length=1)
    action: StrictStr
    
    _check_sg_ids = field_validator('security_group_ids')(check_uuids)
    _check_network_interface_id = field_validator('network_interface_id')(check_uuid)
    
    @field_validator('action')
    def check_action(cls, value):
        if value != 'joinSecurityGroup':
            raise ValueError('action must be joinSecurityGroup')
        return value


class BatchUnBindSecurityGroupParam(BaseParamModel):
    network_interface_id: Optional[StrictStr] = Field(alias='networkInterfaceID')
    security_group_ids: list[str] = Field(alias='securityGroupIDs', min_items=1, max_items=10)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    instance_id: StrictStr = Field(alias='instanceID', min_length=1)
    action: StrictStr
    
    _check_sg_ids = field_validator('security_group_ids')(check_uuids)
    _check_network_interface_id = field_validator('network_interface_id')(check_uuid)
    
    @field_validator('action')
    def check_action(cls, value):
        if value != 'leaveSecurityGroup':
            raise ValueError('action must be leaveSecurityGroup')
        return value


class GetSecurityGroupParam(BaseParamModel):
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    sg_rule_direction: StrictStr = Field('all', alias='direction')
    
    @field_validator('sg_rule_direction')
    def check_sg_rule_direction(cls, value):
        if value not in ('all', 'egress', 'ingress'):
            raise ValueError('only support all / egress / ingress')
        return value


class UpdateSecurityGroupParam(BaseParamModel):
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    name: Optional[str] = Field('', max_length=32, min_length=1)
    description: Optional[str] = Field('', min_length=0, max_length=128)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    enabled: Optional[StrictBool] = None
    
    _check_description = field_validator('description')(check_description)
    _check_name = field_validator('name')(check_name)


class ListSecurityGroupsParam(PaginationGetParamModel):
    vpc_id: StrictStr = Field('', alias='vpcID')
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    instance_id: StrictStr = Field('', alias='instanceID')
    query_content: StrictStr = Field('', alias='queryContent')


class RemoveSecurityGroupRuleParam(BaseParamModel):
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    security_group_rule_id: StrictStr = Field(alias='securityGroupRuleID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)


class BatchRemoveSecurityGroupRuleParam(BaseParamModel):
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    security_group_rule_ids: list[StrictStr] = Field(alias='securityGroupRuleIDs', max_items=10, min_items=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)


class SecurityRule(BaseParamModel):
    direction: StrictStr
    action: StrictStr
    priority: conint(strict=True, ge=1, le=100) = 100
    protocol: StrictStr
    ethertype: StrictStr
    remote_type: StrictInt = Field(0, alias="remoteType")
    remote_ip_prefix: StrictStr = Field('', alias='destCidrIp')
    description: Optional[StrictStr]
    range: StrictStr = Field('')
    remote_security_group_id: Optional[StrictStr] = Field(alias="remoteSecurityGroupID")
    prefixlist_id: Optional[StrictStr] = Field(alias="prefixListID")
    
    _check_description = field_validator('description')(check_description)
    
    @field_validator("remote_type")
    def check_remote_type(cls, v):
        if v is not None and v not in (0, 1, 2):
            raise ValueError("remoteType only support 0, 1, 2")
        return v
    
    @field_validator('direction')
    def check_direction(cls, value: StrictStr):
        if value.lower() not in ('egress', 'ingress'):
            raise ValueError('only support egress / ingress')
        return value.lower()
    
    @field_validator('action')
    def check_action(cls, value: StrictStr):
        tmp = value.lower()
        if tmp not in ('accept', 'drop'):
            raise ValueError('only support accept / drop')
        return tmp
    
    @field_validator('ethertype')
    def check_ethertype(cls, value: StrictStr):
        if value.lower() not in ('ipv4', 'ipv6'):
            raise ValueError('only support ipv4 / ipv6')
        return value.lower()
    
    @field_validator('protocol')
    def check_protocol(cls, value: StrictStr):
        if value.lower() not in ('any', 'tcp', 'udp', 'icmp', 'icmp6'):
            raise ValueError('only support any / tcp / udp / icmp / icmp6')
        return value.lower()
    
    @field_validator('range')
    def check_range(cls, v: StrictStr, values, **kwargs):
        if 'protocol' not in values:
            return v
        
        if values['protocol'].lower() not in ('tcp', 'udp'):
            return ''
        else:
            if not v:
                raise ValueError("when protocol is tcp / udp，range is required")
            
            if v.startswith('-'):
                raise ValueError('range could not start with -')
            tmp = v.split('-')
            if len(tmp) == 2:
                if not tmp[0].isdigit() or not tmp[1].isdigit():
                    raise ValueError('range format support number-number')
                
                if int(tmp[0]) > int(tmp[1]):
                    raise ValueError('end port number must be bigger than start port number')
                
                if int(tmp[0]) < 1 or int(tmp[1]) > 65535:
                    raise ValueError("when protocol is TCP / UDP, port number must be 1 - 65535")
            else:
                if not tmp[0].isdigit():
                    raise ValueError('port number is not digit')
                
                if not 1 <= int(tmp[0]) <= 65535:
                    raise ValueError("when protocol is TCP / UDP, port number must be 1 - 65535")
            return v


class PreCreateCheckParam(BaseParamModel):
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    security_group_rule: SecurityRule = Field(alias='securityGroupRule')


class CreateSecurityGroupRuleParam(BaseParamModel):
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    security_group_rules: List[SecurityRule] = Field(alias='securityGroupRules', min_items=1)


class ModifySecurityGroupRuleParam(BaseParamModel):
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    security_group_rule_id: StrictStr = Field(alias='securityGroupRuleID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    description: Optional[StrictStr]
    action: Optional[StrictStr]
    priority: Optional[conint(strict=True, ge=1, le=100)]
    protocol: Optional[StrictStr]
    remote_type: StrictInt = Field(0, alias="remoteType")
    remote_ip_prefix: Optional[StrictStr] = Field('', alias='destCidrIp')
    remote_security_group_id: Optional[StrictStr] = Field(alias="remoteSecurityGroupID")
    prefixlist_id: Optional[StrictStr] = Field(alias="prefixListID")
    
    _check_description = field_validator('description')(check_description)
    
    @field_validator('action')
    def check_action(cls, value: StrictStr):
        if not value:
            return value
        tmp = value.lower()
        if tmp not in ('accept', 'drop'):
            raise ValueError('only support accept / drop')
        return tmp
    
    @field_validator('protocol')
    def check_protocol(cls, value: StrictStr):
        if not value:
            return value
        if value.lower() not in ('any', 'tcp', 'udp', 'icmp', 'icmp6'):
            raise ValueError('only support any / tcp / udp / icmp / icmp6')
        return value.lower()
    
    @field_validator("remote_type")
    def check_remote_type(cls, v):
        if v is not None and v not in (0, 1, 2):
            raise ValueError("remoteType only support 0, 1, 2")
        return v


class BatchAttachSecurityGroupPortsParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    port_ids: list[str] = Field(alias='portIDs', min_items=1, max_items=10)


class BatchDetachSecurityGroupPortsParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    port_ids: list[str] = Field(alias='portIDs', min_items=1, max_items=10)


class GetSecurityGroupAssociateVmParam(PaginationGetParamModel):
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)


class ShowSecurityGroupRuleParam(PaginationGetParamModel):
    security_group_rule_id: StrictStr = Field(alias='securityGroupRuleID', min_length=1)
    security_group_id: StrictStr = Field(alias='securityGroupID', min_length=1)
    region_id: StrictStr = Field(alias='regionID', min_length=1)



from pyctyun.auth import Auth
from pyctyun.transport import Transport


class CtapiBaseClient(object):
    
    def __init__(self,
                 ak: str,
                 sk: str,
                 endpoint: str = ''):
        """Init.
        :param str ak: access key
        :param str sk: secret key
        :param str endpoint: Api hostname
        """
        self._auth = Auth(ak=ak,
                          sk=sk)
        self._transport = Transport(endpoint=endpoint)
    
    @property
    def transport(self):
        return self._transport

    def perform_request(self, api: str, params: dict, method: str = 'POST', headers=None, timeout=1):
        if method == 'POST':
            sign_headers = self.generate_sign_headers(body_params=params)
        else:
            sign_headers = self.generate_sign_headers(query_params=params)
        if headers:
            sign_headers.update(headers)
        return self._transport.perform_request(api, params=params, method=method, headers=sign_headers, timeout=timeout)
    
    def generate_sign_headers(self, query_params: dict = None, body_params: dict = None):
        return self._auth.generate_sign_headers(query_params=query_params, body_params=body_params)



import logging

from dataclasses import dataclass
from typing import Optional

from src import exceptions
from src.apis.base import CtapiBaseClient

logger = logging.getLogger('')

@dataclass
class CheckUserPermissionParam:
    
    action: str
    user_id: str
    account_id: str
    region_id: Optional[str] = None
    project_id: Optional[str] = None


class IamApi(CtapiBaseClient):

    def check_user_permission(self, param: CheckUserPermissionParam, headers=None, timeout=1):
        params = {
            "action": param.action,
            "assumeUserId": param.user_id,
            "accountId": param.account_id,
            "regionCode": param.region_id,
            "epId": param.project_id,
        }
        try:
            data = self.perform_request('/v1/perm/validate', params, 'POST', headers=headers, timeout=timeout)
            if data['statusCode'] == 'CTIAM_0005':
                return False
            return True
        except exceptions.CtapiException as e:
            logger.warning(f'ctapi error {str(e)}', exc_info=True)
            return False



import os
import unittest

from pyctyun.apis.network.havip import HavipApi
from pyctyun.param.havip import BindOrUnbindVipParam, ListVipParam


class TestHavipApi(unittest.TestCase):
    client = None
    vpc_id = None
    
    @classmethod
    def setUpClass(cls) -> None:
        cls.client = HavipApi(ak=os.getenv("access_key"),
                              sk=os.getenv("secret_key"),
                              endpoint=os.getenv("endpoint"))
        

    @unittest.skip
    def test_bind(self):
        param = BindOrUnbindVipParam(**{
            "clientToken": "xxxxxxxxx",
            "regionID": "bb9fdb42056f11eda1610242ac110002",
            "resourceType": "VM",
            "haVipID": "xxx",
            "instanceID": "xxx",
            "networkInterfaceID": "xx"
        })
        resp = self.client.bind_havip(param)
        self.assertEqual(800, resp.status_code)
    
    def test_list_vip(self):
        param = ListVipParam(**{
            "regionID": "bb9fdb42056f11eda1610242ac110002",
            "clientToken": "ssss"
        })
        
        resp = self.client.list_havip(param)
        self.assertEqual(800, resp.status_code)
        self.assertEqual(16, len(resp.return_obj))



import os
import unittest

from pyctyun.apis.network.vpc import VpcApi
from pyctyun.param.vpc import ListVPCParam, ShowVPCParam, UpdateVPCParam, CreateVPCParam, UpdateVPCIPv6StatusParam, \
    DeleteVPCParam


class TestVpcApi(unittest.TestCase):
    client = None
    vpc_id = None
    
    @classmethod
    def setUpClass(cls) -> None:
        cls.client = VpcApi(ak=os.getenv("access_key"),
                            sk=os.getenv("secret_key"),
                            endpoint=os.getenv("endpoint"))
        
        resp = cls.client.create_vpc(CreateVPCParam(**{
            "regionID": "bb9fdb42056f11eda1610242ac110002",
            "clientToken": "79fa97e3-c48b-xxxx-9f46-6a13d8163678",
            "name": "vpc-for-test-wowop",
            "CIDR": "192.168.0.0/16",
            "enableIpv6": False
        }))
        assert resp.status_code == 800
        cls.vpc_id = resp.return_obj.vpc_id
    
    def test_list_vpc(self):
        resp = self.client.describe_vpcs(ListVPCParam(**{
            'regionID': 'bb9fdb42056f11eda1610242ac110002'
        }))
        self.assertEqual(800, resp.status_code)
        data = resp.return_obj.model_dump(by_alias=True)
        self.assertIn('totalCount', data)
        self.assertIn('vpcs', data)
        self.assertIn('currentCount', data)
        self.assertIn('totalPage', data)
        
        for vpc in data['vpcs']:
            self.assertIn('vpcID', vpc)
            self.assertIn('name', vpc)
            self.assertIn('description', vpc)
            self.assertIn('CIDR', vpc)
            self.assertIn('ipv6Enabled', vpc)
            self.assertIn('enableIpv6', vpc)
            self.assertIn('ipv6CIDRS', vpc)
            self.assertIn('subnetIDs', vpc)
            self.assertIn('natGatewayIDs', vpc)
            self.assertIn('secondaryCIDRS', vpc)
            self.assertIn('projectID', vpc)
    
    def test_show_vpc(self):
        resp = self.client.show_vpc(ShowVPCParam(**{
            'regionID': 'bb9fdb42056f11eda1610242ac110002',
            'vpcID': self.vpc_id
        }))
        self.assertEqual(800, resp.status_code)
        vpc = resp.return_obj.model_dump(by_alias=True)
        self.assertIn('vpcID', vpc)
        self.assertIn('name', vpc)
        self.assertIn('description', vpc)
        self.assertIn('CIDR', vpc)
        self.assertIn('ipv6Enabled', vpc)
        self.assertIn('enableIpv6', vpc)
        self.assertIn('ipv6CIDRS', vpc)
        self.assertIn('subnetIDs', vpc)
        self.assertIn('natGatewayIDs', vpc)
        self.assertIn('secondaryCIDRS', vpc)
        self.assertIn('projectID', vpc)
        
    def test_update_vpc(self):
        resp = self.client.update_vpc(UpdateVPCParam(**{
            'clientToken': 'sssss',
            'regionID': 'bb9fdb42056f11eda1610242ac110002',
            'vpcID': self.vpc_id,
            'name': 'test-test'
        }))
        self.assertEqual(800, resp.status_code)
        resp = self.client.show_vpc(ShowVPCParam(**{
            'regionID': 'bb9fdb42056f11eda1610242ac110002',
            'vpcID': self.vpc_id
        }))
        self.assertEqual(800, resp.status_code)
        self.assertEqual('test-test', resp.return_obj.name)
        
    def test_update_ipv6_status(self):
        resp = self.client.update_vpc_ipv6(UpdateVPCIPv6StatusParam(**{
            'clientToken': 'sssss',
            'regionID': 'bb9fdb42056f11eda1610242ac110002',
            'vpcID': self.vpc_id,
            'enableIpv6': True
        }))
        self.assertEqual(800, resp.status_code)
    
    @classmethod
    def tearDownClass(cls) -> None:
        if cls.client:
            resp = cls.client.delete_vpc(DeleteVPCParam(**{'vpcID': cls.vpc_id, 'regionID': 'bb9fdb42056f11eda1610242ac110002'}))
            assert resp.status_code == 800



from pydantic import BaseModel, Field

from .base import BaseResponse


class CreateSgIDResponse(BaseModel):
    security_group_id: str = Field(alias="securityGroupID")


class SecurityGroupRule(BaseModel):
    direction: str
    priority: int
    ethertype: str
    protocol: str
    range: str
    dest_cidr_ip: str = Field(alias="destCidrIp")
    description: str
    create_time: str = Field(alias="createTime")
    id: str
    security_group_id: str = Field(alias="securityGroupID")
    action: str
    origin: str
    remote_securityGroup_id: str = Field(alias="remoteSecurityGroupID")
    prefix_list_id: str = Field(alias="prefixListID")


class CreateSecurityGroupResponse(BaseResponse):
    return_obj: CreateSgIDResponse = Field(alias="returnObj")


class ShowSgRuleResponse(BaseResponse):
    return_obj: SecurityGroupRule = Field(alias="returnObj")


class SecurityGroup(BaseModel):
    security_group_name: str = Field(alias="securityGroupName")
    id: str
    vm_num: int = Field(alias="vmNum")
    origin: str
    vpc_name: str = Field(alias="vpcName")
    vpc_id: str = Field(alias="vpcID")
    creation_time: str = Field(alias="creationTime")
    description: str
    security_group_rule_list: list[SecurityGroupRule] = Field(alias="securityGroupRuleList")


class NewListSg(BaseModel):
    security_groups: list[SecurityGroup] = Field(alias="securityGroups")
    total_count: int = Field(alias="totalCount")
    current_count: int = Field(alias="currentCount")
    total_page: int = Field(alias="totalPage")


class NewListSgResponse(BaseResponse):
    return_obj: NewListSg = Field(alias="returnObj")


class ListSgResponse(BaseResponse):
    total_count: int = Field(alias="totalCount")
    current_count: int = Field(alias="currentCount")
    total_page: int = Field(alias="totalPage")
    return_obj: list[SecurityGroup] = Field(alias="returnObj")


class ShowSgResponse(BaseResponse):
    return_obj: SecurityGroup = Field(alias="returnObj")


class AsssociateVm(BaseModel):
    instance_id: str = Field(alias="instanceID")
    instance_name: str = Field(alias="instanceName")
    instance_type: str = Field(alias="instanceType")
    instance_state: str = Field(alias="instanceState")
    private_ip: str = Field(alias="privateIp")
    private_ipv6: str = Field(alias="privateIpv6")


class ListSgAssociateObj(BaseModel):
    total_count: int = Field(alias="totalCount")
    current_count: int = Field(alias="currentCount")
    total_page: int = Field(alias="totalPage")
    results: list[AsssociateVm]


class ListSgAssociateVmResponse(BaseResponse):
    return_obj: ListSgAssociateObj = Field(alias="returnObj")



from typing import List, Optional

from pydantic import conint, Field, StrictStr, field_validator

from pyctyun.rules.rule import BaseParamModel, check_description, check_dns_name, check_proxy_pattern, \
    check_uuid, PaginationGetParamModel


class CreateZoneParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vpc_ids: StrictStr = Field(alias='vpcIDList', min_length=1)
    name: StrictStr = Field(min_length=1)
    description: Optional[StrictStr]
    proxy_pattern: Optional[StrictStr] = Field('zone', alias='proxyPattern')
    ttl: Optional[conint(strict=True, ge=300, le=2147483647)] = Field(300, alias='TTL')
    
    _check_name = field_validator('name')(check_dns_name)
    _check_description = field_validator('description')(check_description)
    _check_proxy_pattern = field_validator('proxy_pattern')(check_proxy_pattern)
    
    @field_validator('vpc_ids')
    def check_vpc_ids(cls, v):
        ary = v.split(',')
        if len(ary) > 5:
            raise ValueError("only support 5 vpcs")
        
        for i in ary:
            check_uuid(i)
        return v


class DeleteZoneParam(BaseParamModel):
    client_token: StrictStr = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)


class ShowZoneParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)


class ListZoneParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: Optional[StrictStr] = Field(alias='zoneID')
    zone_name: Optional[StrictStr] = Field(alias='zoneName')


class ModifyZoneParam(BaseParamModel):
    client_token: Optional[StrictStr] = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)
    vpc_ids: Optional[StrictStr] = Field(alias='vpcIDList')
    description: Optional[StrictStr]
    proxy_pattern: Optional[StrictStr] = Field(alias='proxyPattern')
    ttl: Optional[conint(strict=True, ge=300, le=2147483647)] = Field(alias='TTL')
    
    _check_description = field_validator('description')(check_description)
    _check_proxy_pattern = field_validator('proxy_pattern')(check_proxy_pattern)
    
    @field_validator('vpc_ids')
    def check_vpc_ids(cls, v):
        if not v:
            return v
        ary = v.split(',')
        if len(ary) > 5:
            raise ValueError("only support 5 vpcs")
        
        for i in ary:
            check_uuid(i)
        return v


class BindOrUnBindZoneParam(BaseParamModel):
    client_token: Optional[StrictStr] = Field(alias='clientToken', min_length=1, max_length=64)
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)
    vpc_ids: StrictStr = Field(alias='vpcIDList', min_length=1)
    
    @field_validator('vpc_ids')
    def check_vpc_ids(cls, v):
        ary = v.split(',')
        if len(ary) > 5:
            raise ValueError("only support 5 vpcs")
        
        for i in ary:
            check_uuid(i)
        return v


class CheckZoneNameParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_name: StrictStr = Field(alias='zoneName', min_length=1)


class CheckTxtParam(BaseParamModel):
    txt_records: List[StrictStr] = Field(alias='txtRecords', min_items=1, max_items=10, min_length=1)


class CheckMxParam(BaseParamModel):
    mx_records: List[StrictStr] = Field(alias='mxRecords', min_items=1, max_items=10, min_length=1)


class CheckCnameParam(BaseParamModel):
    cname_records: List[StrictStr] = Field(alias='cnameRecords', min_items=1, max_items=10, min_length=1)


class CheckPtrParam(BaseParamModel):
    ptr_records: List[StrictStr] = Field(alias='ptrRecords', min_items=1, max_items=10, min_length=1)


class CheckSrvParam(BaseParamModel):
    srv_records: List[StrictStr] = Field(alias='srvRecords', min_items=1, max_items=10, min_length=1)


class Check4AParam(BaseParamModel):
    aaaa_records: List[StrictStr] = Field(alias='aaaaRecords', min_items=1, max_items=10, min_length=1)


class ListZoneRecordParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: Optional[StrictStr] = Field(alias='zoneID')
    zone_record_id: Optional[StrictStr] = Field(alias='zoneRecordID')


class SetZonePatternParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)
    proxy_pattern: StrictStr = Field(alias='proxyPattern', min_length=1)
    
    _check_proxy_pattern = field_validator('proxy_pattern')(check_proxy_pattern)


class GetZoneLabelsParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)


class ZoneBindLabelParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)
    label_key: StrictStr = Field(alias='labelKey', min_length=1)
    label_value: StrictStr = Field(alias='labelValue', min_length=1)


class ZoneUnBindLabelParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    zone_id: StrictStr = Field(alias='zoneID', min_length=1)
    label_id: StrictStr = Field(alias='labelID', min_length=1)


class GetVpcBindedZonesParam(PaginationGetParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)
    vpc_id: StrictStr = Field(alias='vpcID', min_length=1)


class GetAllZonesAndVpcsParam(BaseParamModel):
    region_id: StrictStr = Field(alias='regionID', min_length=1)



from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param import eip
from pyctyun.response.eip import ListEipResponse, NewListEipResponse


class EipApi(CtapiBaseClient):
    
    def list_eip(self, param: eip.ListEipParam) -> ListEipResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/eip/list', params=params, method='POST')
        return ListEipResponse(**data)
    
    def new_list_eip(self, param: eip.ListEipParam) -> NewListEipResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/eip/new-list', params=params, method='POST')
        return NewListEipResponse(**data)



from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param.monitor import RealtimeMonitorParam
from pyctyun.response.monitor import RealtimeMetricResponse


class MonitorApi(CtapiBaseClient):
    
    def get_realtime_metric(self, param: RealtimeMonitorParam):
        params = param.model_dump(by_alias=True)
        data = self.perform_request("/v4.1/monitor/query-eip-latestmetricdata",
                                    params,
                                    method='POST')
        return RealtimeMetricResponse(**data)



from pyctyun.apis.base import CtapiBaseClient

from pyctyun.param import security_group
from pyctyun.response.base import BaseResponse
from pyctyun.response.security_group import CreateSecurityGroupResponse, ListSgResponse, NewListSgResponse, \
    ShowSgResponse


class SecurityGroupApi(CtapiBaseClient):
    
    def create_security_group(self, param: security_group.CreateSecurityGroupParam) -> CreateSecurityGroupResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/create-security-group', params=params, method='POST')
        return CreateSecurityGroupResponse(**data)
    
    def delete_security_group(self, param: security_group.RemoveSecurityGroupParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/delete-security-group', params=params, method='POST')
        return BaseResponse(**data)
    
    def list_security_group(self, param: security_group.ListSecurityGroupsParam) -> ListSgResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/query-security-groups', params=params, method='GET')
        return ListSgResponse(**data)
    
    def new_list_security_group(self, param: security_group.ListSecurityGroupsParam) -> NewListSgResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/new-query-security-groups', params=params, method='GET')
        return NewListSgResponse(**data)
    
    def update_security_group(self, param: security_group.UpdateSecurityGroupParam) -> BaseResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/modify-security-group-attribute', params=params, method='POST')
        return BaseResponse(**data)
    
    def show_security_group(self, param: security_group.GetSecurityGroupParam) -> ShowSgResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/describe-security-group-attribute', params=params, method='GET')
        return ShowSgResponse(**data)
    
    def bind_security_group(self, param: security_group.BindSecurityGroupParam) -> BaseResponse:
        params = param.model_dump(by_alias=True, exclude_none=True)
        data = self.perform_request('/v4/vpc/join-security-group', params=params, method='POST')
        return BaseResponse(**data)
    
    def unbind_security_group(self, param: security_group.UnbindSecurityGroupParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/leave-security-group', params=params, method='POST')
        return BaseResponse(**data)
    
    def batch_bind_security_group(self, param: security_group.BatchAttachSecurityGroupPortsParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/batch-attach-security-group-ports', params=params, method='POST')
        return BaseResponse(**data)
    
    def batch_unbind_security_group(self, param: security_group.BatchDetachSecurityGroupPortsParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/batch-detach-security-group-ports', params=params, method='POST')
        return BaseResponse(**data)
    
    def port_batch_unbind_sgs(self, param: security_group.BatchUnBindSecurityGroupParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/port-batch-unbind-sgs', params=params, method='POST')
        return BaseResponse(**data)
    
    def create_ingress_rule(self, param: security_group.CreateSecurityGroupRuleParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/create-security-group-ingress', params=params, method='POST')
        return BaseResponse(**data)
    
    def create_egress_rule(self, param: security_group.CreateSecurityGroupRuleParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/create-security-group-egress', params=params, method='POST')
        return BaseResponse(**data)

    def delete_ingress_rule(self, param: security_group.RemoveSecurityGroupRuleParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/revoke-security-group-ingress', params=params, method='POST')
        return BaseResponse(**data)
    
    def delete_egress_rule(self, param: security_group.RemoveSecurityGroupRuleParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/revoke-security-group-egress', params=params, method='POST')
        return BaseResponse(**data)
    
    def update_ingress_rule(self, param: security_group.ModifySecurityGroupRuleParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/modify-security-group-ingress', params=params, method='POST')
        return BaseResponse(**data)

    def update_egress_rule(self, param: security_group.ModifySecurityGroupRuleParam) -> BaseResponse:
        params = param.model_dump(by_alias=True)
        data = self.perform_request('/v4/vpc/modify-security-group-egress', params=params, method='POST')
        return BaseResponse(**data)
